@@ -1,22 +1,5 @@
 /*
- * Authors: Martin Nagy <mnagy@redhat.com>
- *          Adam Tkac   <atkac@redhat.com>
- *
- * Copyright (C) 2008, 2009  Red Hat
- * see file 'COPYING' for use and warranty information
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 or later
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Copyright (C) 2008-2014  bind-dyndb-ldap authors; see COPYING for license
  */
 
 #ifdef HAVE_CONFIG_H
@@ -51,9 +34,9 @@
 #include "ldap_helper.h"
 #include "ldap_convert.h"
 #include "log.h"
-#include "rdlist.h"
 #include "util.h"
 #include "zone_manager.h"
+#include "zone_register.h"
 
 #ifdef HAVE_VISIBILITY
 #define VISIBLE __attribute__((__visibility__("default")))
@@ -198,12 +181,18 @@ detach(dns_db_t **dbp)
 
 /* !!! This could be required for optimizations (like on-disk cache). */
 static isc_result_t
+#if LIBDNS_VERSION_MAJOR < 140
 beginload(dns_db_t *db, dns_addrdatasetfunc_t *addp, dns_dbload_t **dbloadp)
 {
 
 	UNUSED(db);
 	UNUSED(addp);
 	UNUSED(dbloadp);
+#else /* LIBDNS_VERSION_MAJOR >= 140 */
+beginload(dns_db_t *db, dns_rdatacallbacks_t *callbacks) {
+	UNUSED(db);
+	UNUSED(callbacks);
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 
 	fatal_error("ldapdb: method beginload() should never be called");
 
@@ -218,11 +207,17 @@ beginload(dns_db_t *db, dns_addrdatasetfunc_t *addp, dns_dbload_t **dbloadp)
 
 /* !!! This could be required for optimizations (like on-disk cache). */
 static isc_result_t
+#if LIBDNS_VERSION_MAJOR < 140
 endload(dns_db_t *db, dns_dbload_t **dbloadp)
 {
 
 	UNUSED(db);
 	UNUSED(dbloadp);
+#else /* LIBDNS_VERSION_MAJOR >= 140 */
+endload(dns_db_t *db, dns_rdatacallbacks_t *callbacks) {
+	UNUSED(db);
+	UNUSED(callbacks);
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 
 	fatal_error("ldapdb: method endload() should never be called");
 
@@ -230,6 +225,17 @@ endload(dns_db_t *db, dns_dbload_t **dbloadp)
 	return ISC_R_SUCCESS;
 }
 
+#if LIBDNS_VERSION_MAJOR >= 140
+static isc_result_t
+serialize(dns_db_t *db, dns_dbversion_t *version, FILE *file)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_serialize(ldapdb->rbtdb, version, file);
+}
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 
 /* !!! This could be required for optimizations (like on-disk cache). */
 static isc_result_t
@@ -585,14 +591,11 @@ deleterdataset(dns_db_t *db, dns_dbnode_t *node, dns_dbversion_t *version,
 	dns_fixedname_t fname;
 	dns_name_t *zname = NULL;
 	isc_boolean_t empty_node;
-	char attr_name[LDAP_ATTR_FORMATSIZE];
-	dns_rdatalist_t fake_rdlist; /* required by remove_values_from_ldap */
 	isc_result_t result;
 
 	REQUIRE(VALID_LDAPDB(ldapdb));
 
 	dns_fixedname_init(&fname);
-	dns_rdatalist_init(&fake_rdlist);
 	zname = dns_db_origin(ldapdb->rbtdb);
 
 	result = dns_db_deleterdataset(ldapdb->rbtdb, node, version, type,
@@ -610,10 +613,8 @@ deleterdataset(dns_db_t *db, dns_dbnode_t *node, dns_dbversion_t *version,
 		CHECK(remove_entry_from_ldap(dns_fixedname_name(&fname), zname,
 					     ldapdb->ldap_inst));
 	} else {
-		CHECK(rdatatype_to_ldap_attribute(type, attr_name,
-						  LDAP_ATTR_FORMATSIZE));
-		CHECK(remove_attr_from_ldap(dns_fixedname_name(&fname), zname,
-					    ldapdb->ldap_inst, attr_name));
+		CHECK(remove_rdtype_from_ldap(dns_fixedname_name(&fname), zname,
+					    ldapdb->ldap_inst, type));
 	}
 
 cleanup:
@@ -780,7 +781,7 @@ getrrsetstats(dns_db_t *db) {
 }
 #endif /* LIBDNS_VERSION_MAJOR >= 45 */
 
-#if LIBDNS_VERSION_MAJOR >= 82
+#if LIBDNS_VERSION_MAJOR >= 82 && LIBDNS_VERSION_MAJOR < 140
 static isc_result_t
 rpz_enabled(dns_db_t *db, dns_rpz_st_t *st)
 {
@@ -804,7 +805,29 @@ rpz_findips(dns_rpz_zone_t *rpz, dns_rpz_type_t rpz_type,
 	dns_db_rpz_findips(rpz, rpz_type, zone, ldapdb->rbtdb, version,
 			   ardataset, st, query_qname);
 }
-#endif /* LIBDNS_VERSION_MAJOR >= 82 */
+#endif /* LIBDNS_VERSION_MAJOR >= 82 && LIBDNS_VERSION_MAJOR < 140 */
+
+#if LIBDNS_VERSION_MAJOR >= 140
+void
+rpz_attach(dns_db_t *db, dns_rpz_zones_t *rpzs, dns_rpz_num_t rpz_num)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	dns_db_rpz_attach(ldapdb->rbtdb, rpzs, rpz_num);
+}
+
+isc_result_t
+rpz_ready(dns_db_t *db)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_rpz_ready(ldapdb->rbtdb);
+}
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 
 #if LIBDNS_VERSION_MAJOR >= 90
 static isc_result_t
@@ -837,11 +860,36 @@ findext(dns_db_t *db, dns_name_t *name, dns_dbversion_t *version,
 }
 #endif /* LIBDNS_VERSION_MAJOR >= 90 */
 
+#if LIBDNS_VERSION_MAJOR >= 140
+isc_result_t
+setcachestats(dns_db_t *db, isc_stats_t *stats)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_setcachestats(ldapdb->rbtdb, stats);
+}
+
+unsigned int
+hashsize(dns_db_t *db)
+{
+	ldapdb_t *ldapdb = (ldapdb_t *) db;
+
+	REQUIRE(VALID_LDAPDB(ldapdb));
+
+	return dns_db_hashsize(ldapdb->rbtdb);
+}
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
+
 static dns_dbmethods_t ldapdb_methods = {
 	attach,
 	detach,
 	beginload,
 	endload,
+#if LIBDNS_VERSION_MAJOR >= 140
+	serialize, /* see dns_db_serialize(), implementation is not mandatory */
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 	dump,
 	currentversion,
 	newversion,
@@ -880,14 +928,22 @@ static dns_dbmethods_t ldapdb_methods = {
 #endif /* LIBDNS_VERSION_MAJOR >= 50 */
 	getrrsetstats,
 #endif /* LIBDNS_VERSION_MAJOR >= 45 */
-#if LIBDNS_VERSION_MAJOR >= 82
+#if LIBDNS_VERSION_MAJOR >= 82 && LIBDNS_VERSION_MAJOR < 140
 	rpz_enabled,
 	rpz_findips,
-#endif /* LIBDNS_VERSION_MAJOR >= 82 */
+#endif /* LIBDNS_VERSION_MAJOR >= 82 && LIBDNS_VERSION_MAJOR < 140 */
+#if LIBDNS_VERSION_MAJOR >= 140
+	rpz_attach,
+	rpz_ready,
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 #if LIBDNS_VERSION_MAJOR >= 90
 	findnodeext,
-	findext
+	findext,
 #endif /* LIBDNS_VERSION_MAJOR >= 90 */
+#if LIBDNS_VERSION_MAJOR >= 140
+	setcachestats,
+	hashsize
+#endif /* LIBDNS_VERSION_MAJOR >= 140 */
 };
 
 isc_result_t ATTR_NONNULLS
