@@ -1,21 +1,5 @@
 /*
- * Authors: Martin Nagy <mnagy@redhat.com>
- *
- * Copyright (C) 2009-2012  Red Hat
- * see file 'COPYING' for use and warranty information
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 or later
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Copyright (C) 2009-2014  bind-dyndb-ldap authors; see COPYING for license
  */
 
 #include <isc/util.h>
@@ -432,8 +416,8 @@ setting_update_from_ldap_entry(const char *name, settings_set_t *set,
 	result = ldap_entry_getvalues(entry, attr_name, &values);
 	if (result == ISC_R_NOTFOUND || HEAD(values) == NULL) {
 		CHECK(setting_unset(name, set));
-		log_debug(2, "setting '%s' (%s) was deleted in object '%s'",
-			  name, attr_name, entry->dn);
+		log_debug(2, "setting '%s' (%s) was deleted in object %s",
+			  name, attr_name, ldap_entry_logname(entry));
 		return ISC_R_SUCCESS;
 
 	} else if (result != ISC_R_SUCCESS) {
@@ -442,13 +426,14 @@ setting_update_from_ldap_entry(const char *name, settings_set_t *set,
 
 	if (HEAD(values) != TAIL(values)) {
 		log_bug("multi-value attributes are not supported: attribute "
-			"'%s' in entry '%s'", attr_name, entry->dn);
+			"'%s' in %s", attr_name,
+			ldap_entry_logname(entry));
 		return ISC_R_NOTIMPLEMENTED;
 	}
 
 	CHECK(setting_set(name, set, HEAD(values)->value));
-	log_debug(2, "setting '%s' (%s) was changed to '%s' in object '%s'",
-		  name, attr_name, HEAD(values)->value, entry->dn);
+	log_debug(2, "setting '%s' (%s) was changed to '%s' in %s", name,
+		  attr_name, HEAD(values)->value, ldap_entry_logname(entry));
 
 cleanup:
 	if (result == ISC_R_NOTFOUND)
