@@ -1,21 +1,5 @@
 /*
- * Authors: Adam Tkac <atkac@redhat.com>
- *
- * Copyright (C) 2011 Red Hat
- * see file 'COPYING' for use and warranty information
- *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License as
- * published by the Free Software Foundation; version 2 or later
- *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
- *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
+ * Copyright (C) 2011-2013  bind-dyndb-ldap authors; see COPYING for license
  */
 
 #ifndef _LD_TYPES_H_
@@ -48,5 +32,8 @@ typedef struct enum_txt_assoc {
 
 typedef struct ldap_instance	ldap_instance_t;
 typedef struct zone_register	zone_register_t;
+typedef struct mldapdb		mldapdb_t;
+typedef struct ldap_entry	ldap_entry_t;
+typedef struct settings_set	settings_set_t;
 
 #endif /* !_LD_TYPES_H_ */
